module Main exposing (main)

import Html exposing (..)
import Html.Attributes exposing (srcdoc)
import Playground as P exposing (..)
import Tuple exposing (first, second)


type alias Memory =
    { dx : Number
    }


type alias Bounds =
    { ymin : Number
    , ymax : Number
    , xmin : Number
    , xmax : Number
    }


type alias Coords =
    ( Number, Number )


init : Memory
init =
    { dx = 100
    }


main = game


game =
    P.game view update init


view : Computer -> Memory -> List Shape
view computer memory =
    let
        bounds =
            { ymin = 0
            , ymax = 511
            , xmin = 0
            , xmax = 511 * computer.screen.width / computer.screen.height
            }

        at =
            placeAtCoords computer bounds

        fx x =
            (x - (bounds.xmax - bounds.xmin) / 2) ^ 2 / 1000
    in
    List.append
        [ P.words purple "Welcome to my math project! Use arrow keys to adjust the width of each rectangle."
            |> at ( (bounds.xmax - bounds.xmin) / 2, bounds.ymax - 10 )
        , P.words purple "The current equation is y=0.25x^2."
            |> at ( (bounds.xmax - bounds.xmin) / 2, bounds.ymax - 20 )
        , P.words purple ("The current dx is " ++ (String.fromInt <| truncate memory.dx))
            |> at ( (bounds.xmax - bounds.xmin) / 2, bounds.ymax - 30 )
        , P.square purple 10
            |> at ( 100, 100 )
        ]
    <|
        makeRectangles at fx bounds memory []


makeRectangles : (Coords -> Shape -> Shape) -> (Number -> Number) -> Bounds -> Memory -> List Shape -> List Shape
makeRectangles at fx bounds memory prev =
    let
        height =
            fx <| toFloat (List.length prev + 1) * memory.dx

        numOfRecs =
            1 + (truncate <| toFloat (List.length prev) * memory.dx)

        newRec =
            rectangle red memory.dx height
                |> at ( memory.dx / 4 + memory.dx * toFloat (List.length prev) / 2, height / 4 )
    in
    if toFloat (List.length prev) <= 1 + 2 * (bounds.xmax - bounds.xmin) / memory.dx then
        makeRectangles at fx bounds memory <|
            List.append prev [ newRec ]

    else
        prev


update : Computer -> Memory -> Memory
update computer memory =
    { memory
        | dx = memory.dx + 0.2 * toX computer.keyboard
    }


placeAtScreen : Computer -> Coords -> Shape -> Shape
placeAtScreen computer coords shape =
    let
        x =
            first coords

        y =
            second coords
    in
    shape |> P.move (-computer.screen.width / 2 + x) (-computer.screen.height / 2 + y)


placeAtCoords : Computer -> Bounds -> Coords -> Shape -> Shape
placeAtCoords computer bounds coords shape =
    let
        x =
            first coords

        y =
            second coords
    in
    placeAtScreen computer ( x / (bounds.xmax - bounds.xmin) * computer.screen.width, y / (bounds.ymax - bounds.ymin) * computer.screen.height ) shape
